﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Assignment4
{
    public partial class Form1 : Form
    {
        //Initial values
        private readonly static int START_LIVES = 3;
        private readonly static int LEVEL_LENGTH_SECONDS = 30;
        private readonly static int START_OBSTACLES = 5;
        private readonly static int STARS_QUANTITY = 500;
        private readonly static int START_LEVEL = 1;
        private readonly static bool SHOW_DANGERZONES = false;

        //Establish single random object so that the production of stars does not end up using the random object with the same seed (thus placing all 1000 stars on top of one another (more or less))
        private Random random = new Random();
        private HashSet<Star> stars = new HashSet<Star>();
        private HashSet<Obstacle> obstacles = new HashSet<Obstacle>();
        private Ship ship;
        
        private int obstacleQuantity = START_OBSTACLES;

        private int secondsRemaining = LEVEL_LENGTH_SECONDS;
        private int livesRemaining = START_LIVES;
        private int level = START_LEVEL;

        private bool levelRunning = false;
        private bool gameStart = false;
        private bool gameOver = false;
        private bool levelRestart = false;
        private bool levelUp = false;
        private bool crash = false;

        /// <summary>
        /// Default constructor for Form1.
        /// </summary>
        public Form1()
        {
            InitializeComponent();            
        }

        /// <summary>
        /// Initialization of the application. Including creation of objects.
        /// </summary>
        /// <param name="sender">The object which is calling the initialization.</param>
        /// <param name="e">Event arguments.</param>
        private void Form1_Load(object sender, EventArgs e)
        {
            //Start Maximized.
            this.WindowState = FormWindowState.Maximized;

            for (int i = 0; i < STARS_QUANTITY; i++)
            {
                stars.Add(new Star(this.DisplayRectangle, random, true));
            }

            movementTimer.Start();
        }

        /// <summary>
        /// The event used to draw all objects in the drawing area.
        /// </summary>
        /// <param name="sender">The object which paint is being called on.</param>
        /// <param name="e">Event arguments.</param>
        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            foreach (Star star in stars)
            {
                star.Draw(e.Graphics);
            }

            if (levelRunning)
            {
                ship.Draw(e.Graphics);

                foreach (Obstacle obstacle in obstacles)
                {
                    obstacle.Draw(e.Graphics, SHOW_DANGERZONES);
                }

                CheckForCollisions();
            }

            if (!gameStart)
            {
                DisplayWelcome(e.Graphics);
                DisplayPressSpaceToStart(e.Graphics);
            }
            else
            {
                DisplaySecondsRemaining(e.Graphics);
                DisplayLivesRemaining(e.Graphics);
                DisplayLevel(e.Graphics);
            }
            
            if (gameOver)
            {
                DisplayGameOver(e.Graphics);
                DisplayPressSpaceToStart(e.Graphics);
            }

            if (levelRestart)
            {
                DisplayLostLife(e.Graphics);
                DisplayPressSpaceToStart(e.Graphics);
            }

            if (levelUp)
            {
                DisplayLevelUp(e.Graphics);
                DisplayPressSpaceToStart(e.Graphics);
            }

        }

        /// <summary>
        /// A timer used to progress the animation of all animated objects.
        /// </summary>
        /// <param name="sender">The object calling the tick.</param>
        /// <param name="e">Event arguments.</param>
        private void movementTimer_Tick(object sender, EventArgs e)
        {
            foreach(Star star in stars)
            {
                star.Move();
            }

            stars.RemoveWhere(StarOutOfBounds);

            foreach (Obstacle obstacle in obstacles)
            {
                obstacle.Move();
            }

            obstacles.RemoveWhere(ObstacleOutOfBounds);

            Invalidate();
        }

        //TODO: CONSIDER REPLACING WITH 
        //https://msdn.microsoft.com/en-us/library/system.windows.forms.control.mousemove(v=vs.110).aspx
        //private void panel1_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
        //{
        //    // Update the mouse path that is drawn onto the Panel.
        //    int mouseX = e.X;
        //    int mouseY = e.Y;
        //    mousePath.AddLine(mouseX, mouseY, mouseX, mouseY);
        //}

        /// <summary>
        /// Event handler for the KeyDown event.
        /// </summary>
        /// <param name="sender">The object sending the KeyDown event</param>
        /// <param name="e">Key event arguments.</param>
        private void Form1_KeyDown(object sender, KeyEventArgs e)
        {
            switch(e.KeyCode)
            {
                case Keys.Space:
                    {
                        StartLevel();
                        levelRestart = false;
                        levelUp = false;
                        break;
                    }
                case Keys.Escape:
                    {
                        StopLevel();
                        break;
                    }
                case Keys.W:
                    {
                        ship.Move(Ship.Direction.Up);
                        break;
                    }
                case Keys.A:
                    {
                        ship.Move(Ship.Direction.Left);
                        break;
                    }
                case Keys.S:
                    {
                        ship.Move(Ship.Direction.Down);
                        break;
                    }
                case Keys.D:
                    {
                        ship.Move(Ship.Direction.Right);
                        break;
                    }
            }
        }

        /// <summary>
        /// Checks to see if the star is out of bounds and flags for deletion from the stars list. Adds new stars as they are flagged for removal.
        /// </summary>
        /// <param name="star">The star object being checked.</param>
        /// <returns>A boolean indicating whether or not the star is out of bounds.</returns>
        private bool StarOutOfBounds(Star star)
        {
            bool result = star.displayArea.Right < this.DisplayRectangle.Left;

            if (result)
            {
                stars.Add(new Star(this.DisplayRectangle, random, false));
            }

            return result;
        }

        /// <summary>
        /// Check to see if the obstacle is out of bounds and flags for deletion from the obstacles list. Adds new obstacles as they are flagged for removal.
        /// </summary>
        /// <param name="obstacle">The obstacle object being checked.</param>
        /// <returns>A boolean indicating whether or not the obstacle is out of bounds.</returns>
        private bool ObstacleOutOfBounds(Obstacle obstacle)
        {
            bool result = obstacle.displayArea.Right < this.DisplayRectangle.Left;

            if (result)
            {
                obstacles.Add(new Obstacle(this.DisplayRectangle, random, level));
            }

            return result;
        }

        /// <summary>
        /// Timer for tracking the remaining amount of time in the round.
        /// </summary>
        /// <param name="sender">The object calling the tick.</param>
        /// <param name="e">Event arguments.</param>
        private void countDownTimer_Tick(object sender, EventArgs e)
        {
            secondsRemaining--;

            if (secondsRemaining <= 0)
            {
                StopLevel();
                levelUp = true;
                IncreaseLevel();
            }
        }

        /// <summary>
        /// Displays the number of seconds remaining on the screen.
        /// </summary>
        /// <param name="graphics">The object that the graphics are being drawn onto.</param>
        private void DisplaySecondsRemaining(Graphics graphics)
        {
            string message = "Seconds Remaining in Level: {0}";

            Font font = new Font("Arial", 16);
            SolidBrush brush = new SolidBrush(Color.White);
            Point point = new Point(this.DisplayRectangle.Left + 20, this.DisplayRectangle.Top + 20);

            graphics.DrawString(string.Format(message, secondsRemaining), font, brush, point);
        }

        /// <summary>
        /// Displays the number of lives remaining on the screen.
        /// </summary>
        /// <param name="graphics">The object that the graphics are being drawn onto.</param>
        private void DisplayLivesRemaining(Graphics graphics)
        {
            string message = "Lives Remaining: {0}";

            Font font = new Font("Arial", 16);
            SolidBrush brush = new SolidBrush(Color.White);
            Point point = new Point(this.DisplayRectangle.Left + 20, this.DisplayRectangle.Bottom - 40);

            graphics.DrawString(string.Format(message, livesRemaining), font, brush, point);
        }

        /// <summary>
        /// Displays the current level number on the screen.
        /// </summary>
        /// <param name="graphics">The object that the graphics are being drawn onto.</param>
        private void DisplayLevel(Graphics graphics)
        {
            string message = "Level: {0}";

            Font font = new Font("Arial", 16);
            SolidBrush brush = new SolidBrush(Color.White);
            Point point = new Point(this.DisplayRectangle.Right - 100, this.DisplayRectangle.Top + 20);

            graphics.DrawString(string.Format(message, level), font, brush, point);
        }

        /// <summary>
        /// Displays welcome text.
        /// </summary>
        /// <param name="graphics">The object that the graphics are being drawn onto.</param>
        private void DisplayWelcome(Graphics graphics)
        {
            string message = "Welcome to Space Race.";

            Font font = new Font("Arial", 35);
            SolidBrush brush = new SolidBrush(Color.White);
            Point point = new Point(this.DisplayRectangle.Width / 2, this.DisplayRectangle.Height / 2);

            graphics.DrawString(string.Format(message, level), font, brush, point);
        }

        /// <summary>
        /// Displays game over text.
        /// </summary>
        /// <param name="graphics">The object that the graphics are being drawn onto.</param>
        private void DisplayGameOver(Graphics graphics)
        {
            string message = "Game Over!";

            Font font = new Font("Arial", 35);
            SolidBrush brush = new SolidBrush(Color.White);
            Point point = new Point(this.DisplayRectangle.Width / 2 - 250, this.DisplayRectangle.Height / 2);

            graphics.DrawString(string.Format(message, level), font, brush, point);
        }

        /// <summary>
        /// Displays life lost text.
        /// </summary>
        /// <param name="graphics">The object that the graphics are being drawn onto.</param>
        private void DisplayLostLife(Graphics graphics)
        {
            string message = "Be More Careful!";

            Font font = new Font("Arial", 35);
            SolidBrush brush = new SolidBrush(Color.White);
            Point point = new Point(this.DisplayRectangle.Width / 2 - 250, this.DisplayRectangle.Height / 2);

            graphics.DrawString(string.Format(message, level), font, brush, point);
        }

        /// <summary>
        /// Displays level up text.
        /// </summary>
        /// <param name="graphics">The object that the graphics are being drawn onto.</param>
        private void DisplayLevelUp(Graphics graphics)
        {
            string message = "Awesome Flying!";

            Font font = new Font("Arial", 35);
            SolidBrush brush = new SolidBrush(Color.White);
            Point point = new Point(this.DisplayRectangle.Width / 2 - 250, this.DisplayRectangle.Height / 2);

            graphics.DrawString(string.Format(message, level), font, brush, point);
        }

        /// <summary>
        /// Displays instructions to press space bar.
        /// </summary>
        /// <param name="graphics">The object that the graphics are being drawn onto.</param>
        private void DisplayPressSpaceToStart(Graphics graphics)
        {
            string message = "Press space bar to continue.";

            Font font = new Font("Arial", 16);
            SolidBrush brush = new SolidBrush(Color.White);
            Point point = new Point(this.DisplayRectangle.Width / 2, this.DisplayRectangle.Height / 2 + 100);

            graphics.DrawString(string.Format(message, level), font, brush, point);
        }

        /// <summary>
        /// Start the new level
        /// </summary>
        private void StartLevel()
        {
            if (gameOver)
            {
                gameOver = false;
                livesRemaining = START_LIVES;
                level = START_LEVEL;
                obstacleQuantity = START_OBSTACLES;
            }

            ship = new Ship(this.DisplayRectangle);

            for (int i = 0; i < obstacleQuantity; i++)
            {
                obstacles.Add(new Obstacle(this.DisplayRectangle, random, level));
            }

            levelRunning = true;
            gameStart = true;

            countDownTimer.Start();
        }

        /// <summary>
        /// Stop the current level and resets all relevant variables.
        /// </summary>
        private void StopLevel()
        {
            obstacles = new HashSet<Obstacle>();
            secondsRemaining = LEVEL_LENGTH_SECONDS;
            countDownTimer.Stop();
            levelRunning = false;

            //If crash, reduce the lives and restart the level.
            if (crash)
            {
                livesRemaining--;
                levelRestart = true;
                crash = false;
            }

            //If crash and no lives left, show game over.
            if (livesRemaining == 0)
            {
                levelRestart = false;
                gameOver = true;
            }

        }

        /// <summary>
        /// Increments the difficulty of the game.
        /// </summary>
        private void IncreaseLevel()
        {
            level++;
            obstacleQuantity += 2;
        }

        /// <summary>
        /// Checks if ship crashed into obstacle and stops the level as appropriate.
        /// </summary>
        private void CheckForCollisions()
        {
            foreach (Obstacle obstacle in obstacles)
            {
                if (obstacle.displayArea.IntersectsWith(ship.displayArea))
                {
                    crash = true;
                }
            }

            //If a crash is detected, stop the level.
            if (crash)
            {
                StopLevel();
            }
        }

    }
}
