﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Assignment4
{
    class Obstacle
    {
        private int obstacleDiameter;

        private Random random;

        private Rectangle obstacleDisplayArea;
        private Rectangle gamePlayArea;

        private Image image;
        private Color color = new Color();

        public int xVelocity { get; set; }


        /// <summary>
        /// Default constructor.
        /// </summary>
        /// <param name="gamePlayArea">The area that the obstacle is to be drawn onto.</param>
        /// <param name="random">A random object used to generate random locations. The random object is not locally created because it is seeded using the curren time stamp with results in things being not so random.</param>
        /// <param name="level">Whether or not the star should start off screen or on screen.</param>
        public Obstacle(Rectangle gamePlayArea, Random random, int level)
        {
            this.random = random;

            this.gamePlayArea = gamePlayArea;

            //Set maximum obstacle size (10% of the height of the game play area.)
            int maxObsticleSize = Convert.ToInt32(gamePlayArea.Height * 0.3);

            //Set minimum obstacle size (10% of the height of the game play area.)
            int minObsticleSize = Convert.ToInt32(gamePlayArea.Height * 0.1);

            obstacleDiameter = random.Next(minObsticleSize, maxObsticleSize);

            obstacleDisplayArea.Width = obstacleDiameter;
            obstacleDisplayArea.Height = obstacleDiameter;

            //Set maximum speed level based on the current level state of the game. As levels progress the obstacles get faster.
            int levelMaxSpeed;

            if (level > 1)
                {
                levelMaxSpeed = Convert.ToInt32(10 + level * .1);
                }
            else
                {
                    levelMaxSpeed = 10;
                }

            xVelocity = random.Next(1, levelMaxSpeed);

            obstacleDisplayArea.X = random.Next(gamePlayArea.Right, gamePlayArea.Right + (gamePlayArea.Right / 2));
            obstacleDisplayArea.Y = random.Next(gamePlayArea.Top, gamePlayArea.Bottom);

            color = Color.FromArgb(50, 255, 0, 0);
            image = Image.FromFile(string.Format(@"images\{0}.png", random.Next(6)));
        }

        /// <summary>
        /// Move the obstacle's display area XY coordinates.
        /// </summary>
        public void Move()
        {
            obstacleDisplayArea.X -= xVelocity;
        }

        /// <summary>
        /// Draw the obstacle object.
        /// </summary>
        /// <param name="graphics">The object that the graphics are being drawn onto.</param>
        public void Draw(Graphics graphics, Boolean displayDangerZone)
        {
            SolidBrush brush = new SolidBrush(color);
            graphics.DrawImage(image, obstacleDisplayArea);
            if (displayDangerZone)
            {
                graphics.FillRectangle(brush, obstacleDisplayArea); //Leaving out to improve game asthetics.
            }
        }

        /// <summary>
        /// Retrieve the displayArea object for the obstacle.
        /// </summary>
        public Rectangle displayArea
        {
            get { return this.obstacleDisplayArea; }
        }

    }
}
