﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Assignment4
{
    class Ship
    {
        private int shipHeight = 26;
        private int shipWidth = 50;

        private Rectangle shipDisplayArea;
        private Rectangle gamePlayArea;

        private int velocity;

        private Image image;

        public enum Direction { Up, Down, Left, Right };

        /// <summary>
        /// Default constructor.
        /// </summary>
        /// <param name="gamePlayArea">The area that the ship is to appear in.</param>
        public Ship(Rectangle gamePlayArea)
        {
            shipDisplayArea.Height = shipHeight;
            shipDisplayArea.Width = shipWidth;
            this.gamePlayArea = gamePlayArea;

            shipDisplayArea.Y = gamePlayArea.Bottom / 2;
            shipDisplayArea.X = gamePlayArea.Left + shipWidth + 10;

            velocity = Convert.ToInt32(gamePlayArea.Width * 0.005);

            //Set ship image.
            image = Image.FromFile(string.Format(@"images\{0}.png", "ship"));
        }

        /// <summary>
        /// Move the ship's display area XY coordinates, snapping to the sides of the Window.
        /// </summary>
        /// <param name="direction">The direction in which the ship is to move.</param>
        public void Move(Direction direction)
        {
            switch (direction)
            {
                case Direction.Up:
                    {
                        if (shipDisplayArea.Y - shipHeight > gamePlayArea.Top)
                        {
                            shipDisplayArea.Y -= velocity;
                        }
                        else
                        {
                            shipDisplayArea.Y = gamePlayArea.Top;
                        }

                        break;
                    }
                case Direction.Down:
                    {
                        if (shipDisplayArea.Y + shipHeight < gamePlayArea.Bottom - shipHeight)
                        {
                            shipDisplayArea.Y += velocity;
                        }
                        else
                        {
                            shipDisplayArea.Y = gamePlayArea.Bottom - shipHeight;
                        }

                        break;
                    }
                case Direction.Left:
                    {
                        if (shipDisplayArea.X - shipWidth > gamePlayArea.Left)
                        {
                            shipDisplayArea.X -= velocity;
                        }
                        else
                        {
                            shipDisplayArea.X = gamePlayArea.Left;
                        }

                        break;
                    }
                case Direction.Right:
                    {
                        if (shipDisplayArea.X + shipWidth < gamePlayArea.Right - shipWidth)
                        {
                            shipDisplayArea.X += velocity;
                        }
                        else
                        {
                            shipDisplayArea.X = gamePlayArea.Right - shipWidth;
                        }
                        break;
                    }
            }
        }

        /// <summary>
        /// Draw the ship.
        /// </summary>
        /// <param name="graphics">The object that the graphics are being drawn onto.</param>
        public void Draw(Graphics graphics)
        {
            //SolidBrush brush = new SolidBrush(Color.AliceBlue);
            graphics.DrawImage(image, shipDisplayArea);
            //graphics.FillRectangle(brush, shipDisplayArea);
        }

        /// <summary>
        /// Retrieve the displayArea object for the ship.
        /// </summary>
        public Rectangle displayArea
        {
            get { return this.shipDisplayArea; }
        }
    }
}
