﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Assignment4
{
    class Star
    {
        private int starDiameter;

        private Random random;

        private Rectangle starDisplayArea;
        private Rectangle gamePlayArea;

        private Color color = new Color();

        public int XVelocity { get; set; }

        /// <summary>
        /// Default constructor.
        /// </summary>
        /// <param name="gamePlayArea">The area that the star is to be drawn onto.</param>
        /// <param name="random">A random object used to generate random locations. 
        /// The random object is not locally created because it is seeded using the current time stamp which results in things being not so random.</param>
        /// <param name="initial">Whether or not the star should start off screen or on screen.</param>
        public Star(Rectangle gamePlayArea, Random random, Boolean initial)
        {
            this.random = random;

            this.gamePlayArea = gamePlayArea;

            //Set maximum star size (10% of the height of the game play area.)
            int maxStarSize = Convert.ToInt32(gamePlayArea.Height * 0.01);

            starDiameter = random.Next(1, maxStarSize);

            starDisplayArea.Width = starDiameter;
            starDisplayArea.Height = starDiameter;

            //Set maximum star size (10% of the height of the game play area.)
            XVelocity = random.Next(1, starDiameter);

            //If initial is true, stars are randomly placed all over the display area, otherwise, they start offscreen to the right.
            if (initial)
            {
                starDisplayArea.X = random.Next(gamePlayArea.Left, gamePlayArea.Right + (gamePlayArea.Right / 2));
            }
            else
            {
                //Start star in random area off screen right
                starDisplayArea.X = random.Next(gamePlayArea.Right + starDiameter, gamePlayArea.Right + (gamePlayArea.Right / 2));
            }

            starDisplayArea.Y = random.Next(gamePlayArea.Top, gamePlayArea.Bottom);

            //Set random white(ish) color (Not all stars are pure white).
            color = Color.FromArgb(255, random.Next(200, 255), random.Next(200, 255), random.Next(200, 255));

        }

        /// <summary>
        /// Move the star's display area XY coordinates.
        /// </summary>
        public void Move()
        {
            starDisplayArea.X -= XVelocity;
        }

        /// <summary>
        /// Draw the star object.
        /// </summary>
        /// <param name="graphics">The object that the graphics are being drawn onto.</param>
        public void Draw(Graphics graphics)
        {
            SolidBrush brush = new SolidBrush(color);
            graphics.FillEllipse(brush, starDisplayArea);
        }

        /// <summary>
        /// Retrieve the displayArea object for the star.
        /// </summary>
        public Rectangle displayArea
        {
            get { return this.starDisplayArea; }
        }
    }
}
